import re
import time
import thread
import threading
import httplib, urllib
import json
from gi.repository import GObject, Gtk, Gdk, WebKit
Gdk.threads_init()

from login import LoginBackend

def synchronous_gtk_message(fun):
    class NoResult: pass

    def worker((R, function, args, kwargs)):
        R.result = apply(function, args, kwargs)

    def fun2(*args, **kwargs):
        class R: result = NoResult
        GObject.idle_add(worker, (R, fun, args, kwargs))
        while R.result is NoResult: time.sleep(0.01)
        return R.result

    return fun2


def asynchronous_gtk_message(fun):
    def worker((function, args, kwargs)):
        apply(function, args, kwargs)

    def fun2(*args, **kwargs):
        GObject.idle_add(worker, (fun, args, kwargs))

    return fun2


def cookie_is_valid(cookie):
    conn = httplib.HTTPConnection("contribware.caixamagica.pt")
    headers={"Cookie": cookie,
             "Connection": "keep-alive",
         "User-Agent": WebKit.WebSettings().get_property("user_agent"),
         }
    conn.request("GET", "/reviews/session", headers=headers)
    res = conn.getresponse()
    data = json.loads(res.read())
    if data.has_key("username"):
        return data["username"]
    return None

class ContribwareLogin(LoginBackend):

    LOGIN_FAIL_RE = '<div class="errorMsg">'
    LOGIN_OK_RE = '<div class="login_true">'
    def __init__(self):
        LoginBackend.__init__(self)
        self.login_success = False
        self.cookie = None

    def start_thread(self):
        GObject.idle_add(self.run_widget)

    def run_widget(self):
        self.webview = WebKit.WebView()
        self.window = Gtk.Window()
        self.window.connect("destroy", self.quit)

        scroll_w = Gtk.ScrolledWindow()
        scroll_w.add(self.webview)
        scroll_w.show_all()

        self.window.add(scroll_w)

        self.window.set_title("Contribware login")
        self.window.set_default_size(950,500)
        #self.window.set_decorated(False)

        self.webview.connect("load-started", self._load_started)
        self.webview.connect("load-finished", self._load_finished)

        self.window.show_all()
        self.webview.load_uri("http://contribware.caixamagica.pt/account/login")
        self.donebuild = True
    
    def quit(self, widget=None, event=None):
        if self.login_success:
            # login was successful, destroy window
            self.window.destroy()
        else:
            # user clicked on the quit button
            self.emit("login-canceled")
        self.webview.destroy()
        if __name__ == "__main__":
            Gtk.main_quit()

    def _load_started(self, view, frame):
        pass

    def _load_finished(self, view, frame):
        html = self.get_html()

        resp = frame.get_network_response()
        tmp_cookie = resp.get_property("cookie")
        if tmp_cookie:
            self.cookie = tmp_cookie.split(";")[0]

        if self.cookie:
            username = cookie_is_valid(self.cookie)
        if username:
            self.login_success = True
            self.emit("login-successful", (username, self.cookie))
            self.quit()
        """
        if html.find(self.LOGIN_OK_RE) != -1:
            resp = frame.get_network_response()
            cookie = resp.get_property("cookie")
            result = re.search('<p>Ol(?:&aacute;|\xc3\xa1) <strong>([\w._-]+?)</strong></p>', html)
            if result:
                username = result.groups()[0]
            else:
                username = ""
            self.login_success = True
            self.emit("login-successful", (username, cookie))
            self.quit()
        elif html.find(self.LOGIN_FAIL_RE) != -1:
            self.emit("login-failed")
        """


    def get_html(self):
        # really really ugly hack...!
        # http://code.google.com/p/pywebkitgtk/wiki/HowDoI
        self.webview.execute_script('oldtitle=document.title; document.title=document.documentElement.innerHTML;');
        html = self.webview.get_main_frame().get_title()
        self.webview.execute_script('document.title=oldtitle')
        return html


if __name__ == "__main__":
    gtk_t = threading.Thread()
    gtk_t.run = Gtk.main
    gtk_t.start()
    browser = ContribwareLogin()
    browser.start_thread()
    while gtk_t.is_alive():
        time.sleep(0.2)
